<?php

/**
 * Weekly Plugin
 *
 *
 * Created: 2007-03-06
 * Last update: 2007-03-15
 *
 * @link http://deboutv.free.fr/mantis/
 * @author DEBOUT Vincent  <deboutv@free.fr>
 */

$t_core_path = config_get( 'core_path' );

define( 'WEEKLY_HTML', 1 );
define( 'WEEKLY_TEXT', 0 );
define( 'WEEKLY_TEXT_HTML', 2 );

define( 'PLUGINS_WEEKLY_ENABLE_DEFAULT', OFF );
define( 'PLUGINS_WEEKLY_HTML_ENABLE_DEFAULT', OFF );
define( 'PLUGINS_WEEKLY_FROM_HOST_DEFAULT', '' );
define( 'PLUGINS_WEEKLY_FORMAT_DEFAULT', WEEKLY_TEXT );
define( 'PLUGINS_WEEKLY_SUBJECT_TITLE_DEFAULT', 'Bugtracker weekly report' );
define( 'PLUGINS_WEEKLY_ENABLE_DEFAULT', 1 );
define(' PLUGINS_WEEKLY_PERIOD_DEFAULT', 7 );
define( 'PLUGINS_WEEKLY_BUG_NUMBER_DEFAULT', 20 );
define( 'PLUGINS_WEEKLY_EXCLUDED_BUGS_DEFAULT', '' );

define( 'PLUGINS_WEEKLY_UNASSINGED_DEFAULT', '1,on' );
define( 'PLUGINS_WEEKLY_RESOLVED_DEFAULT', '2,on' );
define( 'PLUGINS_WEEKLY_RECENTLY_MODIFIED_DEFAULT', '3,on' );
define( 'PLUGINS_WEEKLY_FEEDBACK_DEFAULT', '0,on' );
define( 'PLUGINS_WEEKLY_VERIFY_DEFAULT', '0,on' );

require_once( $t_core_path . 'current_user_api.php' );
require_once( $t_core_path . 'bug_api.php' );
require_once( $t_core_path . 'string_api.php' );
require_once( $t_core_path . 'date_api.php' );
require_once( $t_core_path . 'icon_api.php' );

function plugins_weekly_get_bugs( &$p_send, $p_start, $p_end, $p_type, $p_project_id, $p_format, $p_use_date ) {
    $t_per_page = config_get( 'plugins_weekly_bug_number', 20, NO_USER, $p_project_id );
    $t_filter = array(
                      'show_category' => array ( '0' => META_FILTER_ANY ),
                      'show_severity' => array ( '0' => META_FILTER_ANY ),
                      'show_status' => array ( '0' => META_FILTER_ANY ),
                      'highlight_changed' => config_get( 'default_show_changed' ),
                      'reporter_id' => array ( '0' => META_FILTER_ANY ),
                      'handler_id' => array ( '0' => META_FILTER_ANY ),
                      'show_resolution' => array ( '0' => META_FILTER_ANY ),
                      'show_build' => array ( '0' => META_FILTER_ANY ),
                      'show_version' => array ( '0' => META_FILTER_ANY ),
                      'hide_status' => array ( '0' => META_FILTER_NONE ),
                      'user_monitor' => array ( '0' => META_FILTER_ANY ),
                      'per_page' => $t_per_page );

    $t_bug_resolved_status_threshold = config_get( 'bug_resolved_status_threshold' );
    $t_update_bug_threshold = config_get( 'update_bug_threshold' );
    $t_hide_status_default = config_get( 'hide_status_default' );
    
    switch( $p_type ) {
      case 'resolved':
        $t_filter['show_status'] = array ( '0' => $t_bug_resolved_status_threshold );
        $t_filter['hide_status'] = array ( '0' => $t_hide_status_default );
        $t_url_link_parameters = 'show_status=' . $t_bug_resolved_status_threshold . '&amp;hide_status=' . $t_bug_resolved_status_threshold;
        break;
      case 'unassigned':
        $t_filter['hide_status'] = array ( '0' => $t_hide_status_default );
        $t_filter['handler_id'] = array ( '0' => META_FILTER_NONE );
        $t_url_link_parameters = 'handler_id=[none]&amp;hide_status=' . $t_hide_status_default;
        break;
      case 'feedback':
        $t_filter['show_status'] = array( '0' => FEEDBACK );
        $t_url_link_parameters = 'show_status=' . FEEDBACK;
        break;
      case 'verify':
        $t_filter['show_status'] = array( '0' => $t_bug_resolved_status_threshold );
        $t_url_link_parameters = 'show_status=' . $t_bug_resolved_status_threshold;
        break;
      default:
        $t_url_link_parameters = 'hide_status=none';
        break;
    }
    $t_filter['do_filter_by_date'] = $p_use_date;
    $t_filter['start_month'] = (int)date( 'n', $p_start );
    $t_filter['start_day'] = (int)date( 'j', $p_start );
    $t_filter['start_year'] = (int)date( 'Y', $p_start );
    $t_filter['end_month'] = (int)date( 'n', $p_end );
    $t_filter['end_day'] = (int)date( 'j', $p_end );
    $t_filter['end_year'] = (int)date( 'Y', $p_end );

    $t_page_number = 1;
    $t_path = str_replace( '/plugins/weekly', '', config_get( 'path' ) );
    $t_icon_path = config_get( 'icon_path' );

    $t_rows = filter_get_bug_rows ( $t_page_number, $t_per_page, $t_page_count, $t_bug_count, $t_filter, $p_project_id );

    $t_excluded_bugs = ',' . config_get( 'plugins_weekly_excluded_bugs', PLUGINS_WEEKLY_EXCLUDED_BUGS_DEFAULT, NO_USER, $p_project_id ) . ',';
    if ( $t_excluded_bugs != ',,' ) {
        for( $i=0; $i<sizeof( $t_rows ); $i++ ) {
            if ( eregi( ',' . $t_rows[$i]['id'] . ',', $t_excluded_bugs ) ) {
                unset( $t_rows[$i] );
            }
        }
    }   
    
    if ( sizeof( $t_rows ) > 0 ) {
        $p_send = true;
    }

    $t_box_title = lang_get( 'my_view_title_' . $p_type );

    if ( $p_format == 'html' ) {
        $t_result = '<table class="width100" cellspacing="1">' . "\n";
        $t_result .= "<tr>\n<td class=\"form-title\" colspan=\"2\">\n";
        $t_result .= '<a class="subtle" href="' . $t_path . 'view_all_set.php?type=1&amp;temporary=y&amp;' . $t_url_link_parameters . '">';
        $t_result .= $t_box_title . '</a>' . ' [';
        $t_result .= '<a class="subtle" href="' . $t_path . 'view_all_set.php?type=1&amp;temporary=y&amp;' . $t_url_link_parameters . '" target="_blank">';
	$t_result .= '^' . '</a>]';
        if ( sizeof( $t_rows ) > 0 ) {
            $v_start = $t_filter['per_page'] * ( $t_page_number - 1 ) + 1;
            $v_end   = $v_start + sizeof( $t_rows ) - 1;
        } else {
            $v_start = 0;
            $v_end   = 0;
        }
        $t_result .= " ($v_start - $v_end / $t_bug_count)";
        $t_result .= "\n</td>\n</tr>\n";
	foreach( $t_rows as $t_row ) {
            if ( !is_array( $t_row ) ) {
                break;
            }
            extract( $t_row, EXTR_PREFIX_ALL, 'v' );

            $t_summary = string_attribute( $v_summary );
            $t_last_updated = date( config_get( 'normal_date_format' ), $v_last_updated );

            $t_status_color = get_status_color( $v_status );

            $t_attachment_count = 0;
            if (  ( file_can_view_bug_attachments( $v_id ) ) ) {
                $t_attachment_count = file_bug_attachment_count( $v_id );
            }

            $t_project_name = project_get_field( $v_project_id, 'name' );
            $t_result .= "<tr bgcolor=\"$t_status_color\">\n";
            $t_result .= "<td class=\"center\" valign=\"top\" width =\"0\" nowrap>\n";
            $t_result .= '<span class="small">';
            $t_result .= string_get_bug_view_link( $v_id, null, true, true ) . '<br />';

            if ( !bug_is_readonly( $v_id ) && access_has_bug_level( $t_update_bug_threshold, $v_id ) ) {
                $t_result .= '<a href="' . $t_path . string_get_bug_update_url( $v_id ) . '"><img border="0" src="' . $t_icon_path . 'update.png' . '" alt="' . lang_get( 'update_bug_button' ) . '" /></a>';
            }
            $t_result .= icon_get_status_icon( $v_priority );
            if ( 0 < $t_attachment_count ) {
                $t_result .= '<a href="' . $t_path . string_get_bug_view_url( $v_id ) . '#attachments">';
                $t_result .= '<img border="0" src="' . $t_icon_path . 'attachment.png' . '"';
                $t_result .= ' alt="' . lang_get( 'attachment_alt' ) . '"';
                $t_result .= ' title="' . $t_attachment_count . ' ' . lang_get( 'attachments' ) . '"';
                $t_result .= ' />' . '</a>';
            }
            if ( VS_PRIVATE == $v_view_state ) {
                $t_result .= '<img src="' . $t_icon_path . 'protected.gif" width="8" height="15" alt="' . lang_get( 'private' ) . '" />';
            }
            $t_result .= '</span></td>';
            $t_result .= '<td class="left" valign="top" width="100%"><span class="small">';
            $t_result .= $t_summary;
            $t_result .= "<br />\n";
            if ( ON == config_get( 'show_bug_project_links' ) && $p_project_id == 0 ) {
                $t_result .= '[' . $t_project_name . '] ';
            }
            $t_result .= string_display( $v_category );
            if ( $v_last_updated > strtotime( '-'.$t_filter['highlight_changed'].' hours' ) ) {
                $t_result .= ' - <b>' . $t_last_updated . '</b>';
            } else {
                $t_result .= ' - ' . $t_last_updated;
            }
            $t_result .= '</span></td></tr>';
	}
        $t_result .= '</table>';
    } else {
        $t_title = $t_box_title;
        $t_link = '<' . $t_path . 'view_all_set.php?type=1&amp;temporary=y&amp;' . $t_url_link_parameters . '>';
        if ( sizeof( $t_rows ) > 0 ) {
            $v_start = $t_filter['per_page'] * ( $t_page_number - 1 ) +1;
            $v_end   = $v_start + sizeof( $t_rows ) -1;
        } else {
            $v_start = 0;
            $v_end   = 0;
        }
        $t_title .= '    (' . $v_start . ' - ' . $v_end . ' / ' . $t_bug_count . ')';
        $t_result = "\n" . $t_title . "\n" . $t_link . "\n";
        $t_result .= str_pad( '', max( strlen( $t_title ), strlen( $t_link ) ), '-' ) . "\n";
	foreach( $t_rows as $t_row ) {
            if ( !is_array( $t_row ) ) {
                break;
            }
            extract( $t_row, EXTR_PREFIX_ALL, 'v' );

            $t_last_updated = date( config_get( 'normal_date_format' ), $v_last_updated );

            $t_project_name = project_get_field( $v_project_id, 'name' );
            $t_result .= ' * ' . bug_format_id( $v_id ) . ': ';
            $t_result .= $v_summary . "\n";
            $t_result .= '            ';
            if ( ON == config_get( 'show_bug_project_links' ) && $p_project_id == 0 ) { 
                $t_result .= '[' . $t_project_name . '] ';
            }           
            $t_result .= string_display( $v_category );
            $t_result .= ' - ' . $t_last_updated . "\n";
            $t_result .= '   <' . $t_path . string_get_bug_view_url( $v_id ) . ">\n\n";
	}
        $t_result .= "\n";
    }
    
    return $t_result;
}

function weekly_get_body( &$p_send, $p_start, $p_end, $p_format, $p_project_id ) {
    $t_project_id = $p_project_id;
    $t_boxes = config_get( 'plugins_weekly_view_boxes', array( 'unassigned' => '1,on', 'resolved' => '2,on', 'recent_mod' => '3,on', 'feedback' => '0,on', 'verify' => '0,on' ), NO_USER, $t_project_id );
    asort( $t_boxes );
    reset( $t_boxes );
    $t_title = config_get( 'plugins_weekly_subject_title', 'Bugtracker weekly report', NO_USER, $t_project_id );
    
    if ( $p_format == 'html' ) {
        $t_result = '<html>' . "\n" . '  <head>' . "\n";
        $t_result .= '    <meta http-equiv="Content-type" content="text/html;charset=utf-8" />' . "\n";
        $t_result .= '    <style type="text/css">' . "\n";
        $t_result .= '      body { background-color: #ffffff; color: #000000; font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt; margin-left: 4px; margin-right: 4px; margin-top: 6px; margin-bottom: 6px; }' . "\n";
        $t_result .= '      address { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 8pt }' . "\n";
        $t_result .= '      span { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt; }' . "\n";
        $t_result .= '      span.pagetitle { font-size: 12pt; font-weight: bold; text-align: center }' . "\n";
        $t_result .= '      span.small { font-size: 8pt; font-weight: normal; }' . "\n";
        $t_result .= '      table.hide { width: 100%; border: solid 0px #ffffff; }' . "\n";
        $t_result .= '      table.width100 { width: 100%; border: solid 1px #000000; }' . "\n";
        $t_result .= '      td { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt; padding: 4px; text-align: left; }' . "\n";
        $t_result .= '      td.form-title { background-color: #ffffff; color: #000000; font-weight: bold; text-align: left; }' . "\n";
        $t_result .= '      a.subtle { color: blue; text-decoration: none; }' . "\n";
        $t_result .= '      td.center { text-align: center; }' . "\n";
        $t_result .= '      .center { text-align: center; }' . "\n";
        $t_result .= '      td.center { text-align: center; }' . "\n";
        $t_result .= '      td.left { text-align: left; }' . "\n";
        $t_result .= '    </style>' . "\n";
        $t_result .= '    <title>' . $t_title . '</title>' . "\n";
        $t_result .= "  </head>\n  <body>\n";
        $t_result .= '    <div align="center">' . "\n";
        $t_result .= '      <span class="pagetitle">' . $t_title . '</span><br /><br />' . "\n";
        $t_result .= '      <span><b>' . date( config_get( 'short_date_format' ), $p_start ) . ' - ' . date( config_get( 'short_date_format' ), $p_end ) . '</b></span><br /><br />' . "\n";
        $t_result .= '      <table class="hide" border="0" cellspacing="3" cellpadding="0">';
    } else {
        $t_result = $t_title . "\n";
        $t_text = '';
        $t_result .= str_pad( $t_text, strlen( $t_title ), '=' ) . "\n\n";
        $t_result .= date( config_get( 'short_date_format' ), $p_start ) . ' - ' . date( config_get( 'short_date_format' ), $p_end ) . "\n\n";
    }

    $t_number_of_boxes = count ( $t_boxes );
    $t_counter = 0;
    while( list( $t_box_title, $t_box_display ) = each( $t_boxes ) ) {
        $t_box_display = explode( ',', $t_box_display );
        $t_use_date = $t_box_display[1];
        $t_box_display = $t_box_display[0];
        if ( $t_box_display == 0 ) {
            $t_number_of_boxes = $t_number_of_boxes - 1;
        } else {
            $t_counter++;
            if ( 1 == $t_counter%2 ) {
                if ( $p_format == 'html' ) {
                    $t_result .= '<tr><td valign="top" width="50%">';
                }
                $t_result .= plugins_weekly_get_bugs( $p_send, $p_start, $p_end, $t_box_title, $t_project_id, $p_format, $t_use_date );
                if ( $p_format == 'html' ) {
                    $t_result .= '</td>';
                }               
            } elseif ( 0 == $t_counter % 2 ) {
                if ( $p_format == 'html' ) {
                    $t_result .= '<td valign="top" width="50%">';
                }               
                $t_result .= plugins_weekly_get_bugs( $p_send, $p_start, $p_end, $t_box_title, $t_project_id, $p_format, $t_use_date );
                if ( $p_format == 'html' ) {
                    $t_result .= '</td></tr>';
                }               
            }
            if ( $p_format == 'html' && ( $t_counter == $t_number_of_boxes ) && 1 == $t_counter % 2 ) {
                $t_result .= '<td valign="top" width="50%"></td></tr>';
            }
        }
    }

    if ( $p_format == 'html' ) {
        $t_result .= '</table></div>';
        $t_result .= '<hr size="1" />';
        $t_result .= '<table border="0" width="100%" cellspacing="0" cellpadding="0"><tr valign="top"><td>';
        $t_result .= '<address><a href="mailto:' . config_get( 'webmaster_email' ) . '">' . config_get( 'webmaster_email' ) . '</a></address>';
        $t_result .= '</td><td></td></tr></table>';
        $t_result .= "  </body>\n</html>\n";
    }

    return $t_result;
}

?>
